
% Clear workspace
clear all, close all, clc
format compact

% Open serial port
com = serial('COM76');
fopen(com);

% Open binary file for saving inertial data
filename = 'mimu_temp_data.bin';
file = fopen(filename, 'w');

% Flush serial ports
while com.BytesAvailable
    fread(com,com.BytesAvailable,'uint8');
end

% Make sure data is read from the IMUs
%command = [48 19 0 0 67];
%fwrite(com,command,'uint8');
%fread(com,4,'uint8');
command = [53 0 53];
fwrite(com,command,'uint8');
fread(com,4,'uint8');

% Request imu temp data
nr_imus=4;
%nr_imus=32;
header = 40;
rate_divider = 5;
imu_mask = [0 0 0 15];
%imu_mask = [255 255 255 255];
command = [header imu_mask rate_divider+128];
command = [command (sum(command)-mod(sum(command),256))/256 mod(sum(command),256)];
fwrite(com,command,'uint8');
fread(com,4,'uint8');

% Open dummy figure with pushbutton such that logging can be aborted
abort = 0;
figure(1);
uicontrol('style','push','string','Abort data logging','callback','abort=1;');
drawnow

% Logg data until pushbutton pressed
while abort==0
    if com.BytesAvailable>0
        fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
    drawnow
end

% Stop output
fwrite(com,[34 0 34],'uint8');

% Close serial port and file
fclose(com);
fclose(file);
close(gcf)

% Parse data and delete logging file
[temp_data,time_stamps,raw_data]=mimu_temp_parse(filename,uint8(nr_imus));
delete(filename);

% Plot data in SI units
temp_data_double = double(temp_data);
time_double = double(time_stamps);
ind=diff(double(time_stamps))<0; ind = [logical(0) ind];
tmp = zeros(size(time_double));
tmp(ind)=2^32;
tmp = cumsum(tmp);
time_double=time_double+tmp;
time_double = time_double - time_double(1);
figure(1),clf, hold on
plot(repmat(time_double,nr_imus,1)'/64e6,temp_data_double'/340+35)
grid on
title('Temperature');
xlabel('time [s]')
ylabel('temp [C]');

