
% Clear workspace
clear all, clc, close all

% Open serial port
com = serial('COM47');
fopen(com);

read_time = 3;

% Open binary file for saving inertial data
filename = 'imu_data.bin';
file = fopen(filename, 'w');

% Flush serial ports
while com.BytesAvailable
    fread(com,com.BytesAvailable,'uint8');
end

% Turn on inertial front-end preprocessing
header = hex2dec('30');
proc_id = hex2dec('20');
proc_pos = 0;
command = [header proc_id proc_pos];
command = [command (sum(command)-mod(sum(command),256))/256 mod(sum(command),256)];
fwrite(com,command,'uint8');
fread(com,4,'uint8')

% Request combined inertial readings and time stamps
header = hex2dec('21');
state_ids = [hex2dec('01') hex2dec('13') 0 0 0 0 0 0];
mode = 6;
command = [header state_ids mode];
command = [command (sum(command)-mod(sum(command),256))/256 mod(sum(command),256)];
fwrite(com,command,'uint8');
fread(com,4,'uint8')

% Read inertial values for 2 seconds
t_end=cputime+read_time;
while t_end-cputime>0
    if com.BytesAvailable>0
        fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
end

% Store and empty process sequence
command = [hex2dec('37') 0 hex2dec('37')];
fwrite(com,command,'uint8');
fread(com,4,'uint8');

% Read inertial values for 2 seconds
t_end=cputime+read_time;
while t_end-cputime>0
    if com.BytesAvailable>0
        fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
end

% Restore process sequence
command = [hex2dec('38') 0 hex2dec('38')];
fwrite(com,command,'uint8');
fread(com,4,'uint8');

% Read inertial values for 2 seconds
t_end=cputime+read_time;
while t_end-cputime>0
    if com.BytesAvailable>0
        fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
end

% Stop output and processing
fwrite(com,[34 0 34],'uint8');
fread(com,4,'uint8');
fwrite(com,[50 0 50],'uint8');
fread(com,4,'uint8');

% Flush serial ports
while com.BytesAvailable
    fread(com,com.BytesAvailable,'uint8');
end

% Close serial port and file
fclose(com);
fclose(file);

% Parse data and delete logging file
[inertial_data,time_stamps,raw_data]=parse_imu_data(filename);
delete(filename);

% Plot data in SI units
figure(1),clf, hold on
plot(inertial_data(1:3,:)');
plot(sqrt(sum(inertial_data(1:3,:).^2)),'c');
grid on
title('Accelerometer readings');
xlabel('sample number')
ylabel('a [m/s^2]');
figure(2);
plot(inertial_data(4:6,:)'*180/pi);
grid on
title('Gyroscope readings');
xlabel('sample number')
ylabel('\omega [deg/s]');
figure(3),clf, hold on
subplot(2,1,1);
plot(double(time_stamps)'/64e6,'b-');
grid on
title('Time stamps');
xlabel('sample number')
ylabel('[s]');
subplot(2,1,2);
dt = diff(double(time_stamps)');
for i=1:numel(dt)
    if dt(i)<0
        dt(i) = dt(i)+2^32;
    end
end
plot(dt/64e6,'b-');
grid on
title('Time differentials');
xlabel('sample number')
ylabel('[s]');
