
% Clear workspace
clear all, clc, close all

% Open serial port
com = serial('COM6');
fopen(com);

% Open binary file for saving inertial data
filename = 'imu_data.bin';
file = fopen(filename, 'w');

% Turn on frontend
fwrite(com,[53 0 53],'uint8');
fread(com,4,'uint8')

% Request inertial input and output and test statistics
command = [33 16 17 21 22 0 0 0 0 4];
command = [command (sum(command)-mod(sum(command),256))/256 mod(sum(command),256)];
fwrite(com,command,'uint8');
fread(com,4,'uint8')

% Open dummy figure with pushbutton such that logging can be aborted
abort = 0;
figure;
uicontrol('style','push','string','Abort data logging','callback','abort=1;');
drawnow

% Logg data until pushbutton pressed
nr_bytes = 0;
while abort==0
    if com.BytesAvailable>0
        nr_bytes = nr_bytes + fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
    drawnow
end

% Stop output and processing
fwrite(com,[34 0 34],'uint8');
fwrite(com,[50 0 50],'uint8');
fread(com,8,'uint8')

% Close figure, serial port and file
close;
fclose(com);
fclose(file);

% Parse data and delete logging file
[data]=mimu_parse_frontend(filename);
delete(filename);

data_double = double(data);

figure(1),clf, hold on
plot(data_double(7:9,:)'*1.46180391e-7);
plot(sqrt(sum(data_double(7:9,:).^2))*1.46180391e-7,'c');
grid on
title('Accelerometer readings');
xlabel('sample number')
ylabel('a [m/s^2]');
figure(2);
plot(data_double(10:12,:)'*3.24775742e-8*180/pi);
grid on
title('Gyroscope readings');
xlabel('sample number')
ylabel('\omega [deg/s]');
figure(3);
plot(data_double(13,:))
grid on
title('T_1 test statistics');
xlabel('sample number')
ylabel('T_1');
figure(4);
plot(data_double(14,:))
grid on
title('T_2 test statistics');
xlabel('sample number')
ylabel('T_2');


