
% Clear workspace
clear all, clc, close all

% Open serial port
com = serial('COM56');
fopen(com);

% Open binary file for saving inertial data
filename = 'imu_data.bin';
file = fopen(filename, 'w');

% Flush serial ports
while com.BytesAvailable
    fread(com,com.BytesAvailable,'uint8');
end

% Request combined inertial readings in SI-units
header = 64; % 64 = no online bias estimation, 65 = online bias estimation
rate_divider = 4;
command = [header rate_divider];
command = [command (sum(command)-mod(sum(command),256))/256 mod(sum(command),256)];
fwrite(com,command,'uint8');
fread(com,4,'uint8')

% Open dummy figure with pushbutton such that logging can be aborted
abort = 0;
figure;
uicontrol('style','push','string','Abort data logging','callback','abort=1;');
drawnow

% Logg data until pushbutton pressed
nr_bytes = 0;
while abort==0
    if com.BytesAvailable>0
        nr_bytes = nr_bytes + fwrite(file,fread(com,com.BytesAvailable,'uint8'),'uint8');
    end
    drawnow
end

% Stop output
fwrite(com,[34 0 34],'uint8');
fread(com,4,'uint8')

% Close serial port and file
close;
fclose(com);
fclose(file);

% Parse data and delete logging file
[inertial_data,time_stamps,raw_data]=parse_imu_data(filename);
%delete(filename);

% Plot data in SI units
figure(1),clf, hold on
plot(inertial_data(1:3,:)');
plot(sqrt(sum(inertial_data(1:3,:).^2)),'c');
grid on
title('Accelerometer readings');
xlabel('sample number')
ylabel('a [m/s^2]');
figure(2), clf, hold on
plot(inertial_data(4:6,:)'*180/pi);
plot(sqrt(sum((inertial_data(4:6,:)*180/pi).^2)),'c');
grid on
title('Gyroscope readings');
xlabel('sample number')
ylabel('\omega [deg/s]');
figure(3),clf, hold on
subplot(2,1,1);
plot(double(time_stamps)'/64e6,'b-');
grid on
title('Time stamps');
xlabel('sample number')
ylabel('[s]');
subplot(2,1,2);
dt = diff(double(time_stamps)');
for i=1:numel(dt)
    if dt(i)<0
        dt(i) = dt(i)+2^32;
    end
end
plot(dt/64e6,'b-');
grid on
title('Time differentials');
xlabel('sample number')
ylabel('[s]');

figure(4), clf, hold on
plot(sqrt(sum((inertial_data(4:6,:)*180/pi).^2)),'c');
grid on
